/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/filentry.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 03/11/2003
 *
 *  Classe FileEntry d'extension du toolkit GTK+ 1.2.x .
 */


#ifndef SCAN_DEPEND
   #include <string.h>
   #include <gtk/gtkbutton.h>
   #include <gtk/gtkentry.h>
   #include <gtk/gtkfilesel.h>
   #include <gtk/gtklabel.h>
   #include <gtk/gtkmain.h>
   #include <gtk/gtksignal.h>
#endif

#include "linux/filentry.h"



/* partie prive de la classe FileEntry:
 */
enum {
    FILE_SELECTED_SIGNAL,
    LAST_SIGNAL
};

static gint file_entry_signals[LAST_SIGNAL]={0};



static void file_entry_class_init(FileEntryClass *class)
{
    GtkObjectClass *object_class;

    object_class=(GtkObjectClass *) class;

    file_entry_signals[FILE_SELECTED_SIGNAL]=gtk_signal_new("file_selected", GTK_RUN_FIRST, object_class->type, GTK_SIGNAL_OFFSET(FileEntryClass, file_selected), gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

    gtk_object_class_add_signals(object_class, file_entry_signals, LAST_SIGNAL);

    class->file_selected=NULL;
}



static void file_entry_filesel_ok(GtkWidget *button, FileEntry *filent)
{
    strncpy(filent->filename, gtk_file_selection_get_filename( GTK_FILE_SELECTION(filent->filesel)), FILENT_LENGTH);

    gtk_widget_destroy( GTK_WIDGET(filent->filesel));

    /* On envoie le signal file_selected */
    gtk_signal_emit( GTK_OBJECT(filent), file_entry_signals[FILE_SELECTED_SIGNAL]);

    (void) button;
}



static void file_entry_popup_filesel(GtkWidget *button, FileEntry *filent)
{
    filent->filesel=gtk_file_selection_new("Slection d'un fichier");

    /* On passe le nom du fichier au slecteur de fichier */
    gtk_file_selection_set_filename( GTK_FILE_SELECTION(filent->filesel), filent->filename);

    /* Connect the ok_button to filesel_ok function */
    gtk_signal_connect(GTK_OBJECT( GTK_FILE_SELECTION(filent->filesel)->ok_button), "clicked", GTK_SIGNAL_FUNC(file_entry_filesel_ok), filent);

    /* Connect the cancel_button to destroy the widget */
    gtk_signal_connect_object( GTK_OBJECT( GTK_FILE_SELECTION(filent->filesel)->cancel_button), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(filent->filesel));

    gtk_quit_add_destroy(1, GTK_OBJECT(filent->filesel));
    gtk_widget_show(filent->filesel);

    (void) button;
}



static void file_entry_init(FileEntry *filent)
{
    filent->hbox.box.homogeneous=FALSE;
    filent->hbox.box.spacing=5;

    filent->label=gtk_label_new("\0");
    gtk_box_pack_start( GTK_BOX(&filent->hbox), filent->label, FALSE, FALSE,0);
    gtk_widget_show(filent->label);

    filent->entry=gtk_entry_new();
    gtk_entry_set_editable( GTK_ENTRY(filent->entry), FALSE);
    gtk_box_pack_start( GTK_BOX(&filent->hbox), filent->entry, TRUE, TRUE, 0);
    gtk_widget_show(filent->entry);

    filent->button=gtk_button_new_with_label("Parcourir...");
    gtk_signal_connect( GTK_OBJECT(filent->button), "clicked", GTK_SIGNAL_FUNC(file_entry_popup_filesel), filent);
    gtk_box_pack_start( GTK_BOX(&filent->hbox), filent->button, FALSE,FALSE,0);
    gtk_widget_show(filent->button);
}



/* partie publique de la classe FileEntry:
 */
guint file_entry_get_type(void)
{
    static guint filent_type = 0;

    if (!filent_type)
    {
        GtkTypeInfo filent_info={ "FileEntry",
				  sizeof(FileEntry),
				  sizeof(FileEntryClass),
				  (GtkClassInitFunc) file_entry_class_init,
				  (GtkObjectInitFunc) file_entry_init,
				  NULL,
				  NULL,
				  (GtkClassInitFunc) NULL };

        filent_type=gtk_type_unique(gtk_hbox_get_type(),&filent_info);
    }

    return filent_type;
}



GtkWidget *file_entry_new(const gchar *label)
{
    FileEntry *filent=FILE_ENTRY( gtk_type_new (file_entry_get_type() ));

    gtk_label_set_text( GTK_LABEL(filent->label), label);

    return GTK_WIDGET(filent);
}



void file_entry_set_entry(FileEntry *filent, const gchar *text)
{
    gtk_entry_set_text( GTK_ENTRY(filent->entry), text);
}



void file_entry_set_filename(FileEntry *filent, const gchar *filename)
{
    strncpy(filent->filename, filename, FILENT_LENGTH);
}



gchar *file_entry_get_filename(FileEntry *filent)
{
    return filent->filename;
}
