/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/gui.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 03/11/2003
 *
 *  Interface utilisateur de l'mulateur base sur GTK+ 1.2.x .
 */


#ifndef SCAN_DEPEND
   #include <stdio.h>
   #include <stdlib.h>
   #include <string.h>
   #include <unistd.h>
# if 0 //LUDO:
   #include <gdk/gdkx.h>
   #include <gtk/gtk.h>
# endif
#endif

#include "display.h"
#include "filentry.h"
#include "graphic.h"
#include "main.h"
#include "message.h"
#include "question.h"
#include "to7.h"
#include "thomson.xpm"


#define NDISKS       4
#define COUNTER_MAX  999

/* dirty hack... */
struct dummy_pair {
    int num;
    GtkWidget *b;
};

struct dummy_octo {
    GtkWidget *f[4],*b[4];
};

/* fentre de l'interface utilisateur */
static GtkWidget *window;

/* compteur de cassettes */
static GtkWidget *spinner_cass;



/* basename:
 *  Helper pour extraire le nom simple du fichier.
 */
static char *basename(char *filename)
{
    while (*filename)
        filename++;

    while (*filename != '/')
        filename--;

    return filename+1;
}



/* retrace_callback:
 *  Callback de retraage de la fentre principale.
 */
static GdkFilterReturn retrace_callback(GdkXEvent *xevent, GdkEvent *event, gpointer unused)
{
    XEvent *ev = (XEvent *) xevent;

    if (ev->type == Expose)
        RetraceScreen(ev->xexpose.x, ev->xexpose.y, ev->xexpose.width, ev->xexpose.height);

    (void) event;
    (void) unused;

    return GDK_FILTER_REMOVE;
}



static void do_exit(GtkWidget *button, int command)
{
    if (command != NONE)
        thom.command=command;

    gtk_widget_hide(window);
    gtk_main_quit();

    (void) button;
}



static void question_exit(GtkWidget *button, gpointer unused)
{
    GtkWidget *quest=question_new("Voulez-vous vraiment quitter l'mulateur ?");
    gtk_signal_connect( GTK_OBJECT(quest), "clicked_yes", GTK_SIGNAL_FUNC(do_exit), (gpointer) QUIT);
    gtk_quit_add_destroy(1, GTK_OBJECT(quest));
    gtk_widget_show(quest);

    (void) button;
    (void) unused;
}



static void toggle_button_true(GtkWidget *button, gpointer unused)
{
    thom.exact_speed=( GTK_TOGGLE_BUTTON(button)->active ? TRUE : FALSE);

    (void) unused;
}



static void read_filent_memo(GtkWidget *filent, gpointer unused)
{
    char *filename=file_entry_get_filename(FILE_ENTRY(filent));

    if (to7_LoadMemo7(filename) == TO7_ERROR)
    {
        GtkWidget *mesg=message_new(to7_error_msg);
        gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
    }
    else
    {
        file_entry_set_entry( FILE_ENTRY(filent), to7_GetMemo7Label());
        thom.command=COLD_RESET;
    }

    (void) unused;
}



static void read_filent_cass(GtkWidget *filent, GtkToggleButton *button)
{
    int ret=to7_LoadK7( file_entry_get_filename(FILE_ENTRY(filent)) );

    if (ret==TO7_ERROR)
    {
        GtkWidget *mesg=message_new(to7_error_msg);
        gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
    }
    else
    {
        file_entry_set_entry( FILE_ENTRY(filent), basename(file_entry_get_filename(FILE_ENTRY(filent))));

        if ((ret==TO7_READ_ONLY) && !button->active)
        {
            GtkWidget *mesg=message_new("Attention: criture impossible.");
            gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
            gtk_widget_show(mesg);
            gtk_toggle_button_set_active(button, TRUE);
        }
    }
}



static void toggle_check_cass(GtkWidget *button, GtkSpinButton *spinner)
{
    if ( GTK_TOGGLE_BUTTON(button)->active )
        to7_SetK7Mode(TO7_READ_ONLY);
    else if (to7_SetK7Mode(TO7_READ_WRITE)==TO7_READ_ONLY)
    {
        GtkWidget *mesg=message_new("Ecriture impossible sur ce support.");
        gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button), TRUE);
    }

    gtk_spin_button_set_value(spinner, to7_GetK7Counter());
}



static void change_counter_cass(GtkWidget *widget, GtkSpinButton *spinner)
{
   to7_SetK7Counter(gtk_spin_button_get_value_as_int(spinner));

   (void) widget;
}



static void click_rewind_cass(GtkWidget *button, GtkSpinButton *spinner)
{
   to7_SetK7Counter(0);

   gtk_spin_button_set_value(spinner, to7_GetK7Counter());

   (void) button;
}



static void read_filent_disk(GtkWidget *filent, struct dummy_pair *button)
{
    int ret=to7_LoadDisk( button->num, file_entry_get_filename(FILE_ENTRY(filent)) );
   
    if (ret==TO7_ERROR)
    {
        GtkWidget *mesg=message_new(to7_error_msg);
        gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
    }
    else
    {
        file_entry_set_entry( FILE_ENTRY(filent), basename(file_entry_get_filename(FILE_ENTRY(filent))));

        if ((ret==TO7_READ_ONLY) && !(GTK_TOGGLE_BUTTON(button->b)->active) )
        {
            GtkWidget *mesg=message_new("Attention: criture impossible.");
            gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
            gtk_widget_show(mesg);
            gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button->b), TRUE);
        }
    }
}



static void toggle_check_disk(GtkWidget *button, int num)
{
    if ( GTK_TOGGLE_BUTTON(button)->active )
        to7_SetDiskMode(num, TO7_READ_ONLY);
    else if (to7_SetDiskMode(num, TO7_READ_WRITE)==TO7_READ_ONLY)
    {
        GtkWidget *mesg=message_new("Ecriture impossible sur ce support.");
        gtk_quit_add_destroy(1, GTK_OBJECT(mesg));
        gtk_widget_show(mesg);
        gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button), TRUE);
    }
}



static void set_direct_disk(GtkWidget *button, struct dummy_octo *filentry_octo)
{
    int i;

    for (i=0; i<4; i++)
    {
        if (filentry_octo->f[i])
        {
            file_entry_set_entry( FILE_ENTRY(filentry_octo->f[i]), "accs direct");

            if (to7_DirectSetDrive(i) == TO7_READ_ONLY)
                gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(filentry_octo->b[i]), TRUE);
            else
                gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(filentry_octo->b[i]), FALSE);
        }
    }

    (void)button;
}



/* InitGUI:
 *  Initialise le module interface utilisateur.
 */
void InitGUI(const char version_name[], int disk_support, int direct_disk_support)
{
    GtkWidget *vbox_window;
    GtkWidget *hbox_title, *label_title, *pixmapwid1, *pixmapwid2;
    GtkWidget *frame_command, *vbox_command, *button_reset, *button_coldreset;
    GtkWidget *hbox_speed, *label_speed, *button_true, *button_max;
    GtkWidget *notebook;
    GtkWidget *label_memo, *frame_memo, *vbox_memo, *filent_memo;
    GtkWidget *label1_cass, *label2_cass, *frame_cass, *vbox_cass;
    GtkWidget *hbox1_cass, *hbox2_cass, *filent_cass, *check_button_cass;
    GtkWidget *button_rewind;
    GtkWidget *label_disk, *frame_disk, *vbox_disk;
    GtkWidget *hbox_disk[NDISKS], *filent_disk[NDISKS];
    struct dummy_pair *check_button_disk[NDISKS];
    struct dummy_octo *filentry_octo;
    GtkWidget *button_direct;
    GtkWidget *hbox_end, *button_ok, *button_quit;
    GtkObject *adj_cass;
    GtkStyle  *style;
    GdkPixmap *pixmap;
    GdkBitmap *mask;
    GSList *group_speed;
    char string_title[100];
    int i;

    printf("Initialisation de l'interface...");

    /* fentre d'affichage */
    window=gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_window_set_title(GTK_WINDOW(window), "Thom - Panneau de contrle");
    gtk_window_set_policy(GTK_WINDOW(window), False, False, False);
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
    gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(do_exit), (gpointer) QUIT);
    gtk_widget_realize(window); /* ncessaire pour charger le pixmap */

    /* boite verticale associe  la fentre */
    vbox_window=gtk_vbox_new(FALSE,5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_window), 5);
    gtk_container_add( GTK_CONTAINER(window), vbox_window);
    gtk_widget_show(vbox_window);

    /* boite horizontale du titre */
    hbox_title=gtk_hbox_new(FALSE,5);
    gtk_box_pack_start( GTK_BOX(vbox_window), hbox_title, TRUE, FALSE, 5);
    gtk_widget_show(hbox_title);

    /* cration du pixmap */
    style=gtk_widget_get_style(window);
    pixmap=gdk_pixmap_create_from_xpm_d(window->window, &mask, &style->bg[GTK_STATE_NORMAL], thomson_xpm);

    /* premire instance du pixmap */
    pixmapwid1=gtk_pixmap_new(pixmap,mask);
    gtk_box_pack_start( GTK_BOX(hbox_title), pixmapwid1, TRUE, FALSE, 0);
    gtk_widget_show(pixmapwid1);

    /* label du titre */
    strcat(strcpy(string_title,version_name),"\nCopyright  1996 Sylvain Huet,\n1999-2003 Eric Botcazou.");
    label_title=gtk_label_new(string_title);
    gtk_box_pack_start( GTK_BOX(hbox_title), label_title, TRUE, FALSE, 0);
    gtk_widget_show(label_title);

    /* deuxime instance du pixmap */
    pixmapwid2=gtk_pixmap_new(pixmap,mask);
    gtk_box_pack_start( GTK_BOX(hbox_title), pixmapwid2, TRUE, FALSE, 0);
    gtk_widget_show(pixmapwid2);

    /* frame des commandes et rglages */
    frame_command=gtk_frame_new("Commandes et Rglages");
    gtk_box_pack_start( GTK_BOX(vbox_window), frame_command, TRUE, FALSE, 0);
    gtk_widget_show(frame_command);

    /* boite verticale associe  la frame des commandes et rglages */
    vbox_command=gtk_vbox_new(FALSE,5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_command), 5);
    gtk_container_add( GTK_CONTAINER(frame_command), vbox_command);
    gtk_widget_show(vbox_command);

    /* bouton de rinitialisation */
    button_reset=gtk_button_new_with_label("Rinitialiser le TO7-70");
    gtk_signal_connect( GTK_OBJECT(button_reset), "clicked", GTK_SIGNAL_FUNC(do_exit), (gpointer) RESET);
    gtk_box_pack_start( GTK_BOX(vbox_command), button_reset, TRUE, FALSE, 0);
    gtk_widget_show(button_reset);

    /* bouton de redmarrage  froid */
    button_coldreset=gtk_button_new_with_label("Redmarrer  froid le TO7-70");
    gtk_signal_connect( GTK_OBJECT(button_coldreset), "clicked", GTK_SIGNAL_FUNC(do_exit), (gpointer) COLD_RESET);
    gtk_box_pack_start( GTK_BOX(vbox_command), button_coldreset, TRUE, FALSE, 0);
    gtk_widget_show(button_coldreset);

    /* boite horizontale de la vitesse */
    hbox_speed=gtk_hbox_new(FALSE,5);
    gtk_box_pack_start( GTK_BOX(vbox_command), hbox_speed, TRUE, FALSE, 0);
    gtk_widget_show(hbox_speed);

    /* label de la vitesse */
    label_speed=gtk_label_new("Vitesse de l'mulation:");
    gtk_box_pack_start( GTK_BOX(hbox_speed), label_speed, TRUE, TRUE, 0);
    gtk_widget_show(label_speed);

    /* bouton de vitesse maximale */
    button_max=gtk_radio_button_new_with_label(NULL, "maximale");
    gtk_box_pack_end( GTK_BOX(hbox_speed), button_max, TRUE, TRUE, 0);
    gtk_widget_show(button_max);

    /* bouton de vitesse exacte */
    group_speed=gtk_radio_button_group( GTK_RADIO_BUTTON(button_max));
    button_true=gtk_radio_button_new_with_label(group_speed, "exacte");
    gtk_signal_connect( GTK_OBJECT(button_true), "toggled", GTK_SIGNAL_FUNC(toggle_button_true), NULL);
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(button_true), thom.exact_speed ? TRUE : FALSE);
    gtk_box_pack_end( GTK_BOX(hbox_speed), button_true, TRUE, TRUE, 0);
    gtk_widget_show(button_true);

    /* notebook des priphriques */
    notebook=gtk_notebook_new();
    gtk_box_pack_start( GTK_BOX(vbox_window), notebook, TRUE, FALSE, 0);
    gtk_widget_show(notebook);

    /* frame de la cartouche */
    frame_memo=gtk_frame_new("");
    gtk_frame_set_shadow_type( GTK_FRAME(frame_memo), GTK_SHADOW_NONE);
    gtk_frame_set_label_align( GTK_FRAME(frame_memo), 0.985, 0.0);
    label_memo=gtk_label_new("Cartouche");
    gtk_notebook_append_page( GTK_NOTEBOOK(notebook), frame_memo, label_memo);
    gtk_widget_show(frame_memo);
    gtk_widget_show(label_memo);

    /* boite verticale associe  la frame de la cartouche */
    vbox_memo=gtk_vbox_new(FALSE,0);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_memo), 5);
    gtk_container_add( GTK_CONTAINER(frame_memo), vbox_memo);
    gtk_widget_show(vbox_memo);

    /* entre fichier de la cartouche */
    filent_memo=file_entry_new("m7");
    gtk_signal_connect( GTK_OBJECT(filent_memo), "file_selected", GTK_SIGNAL_FUNC(read_filent_memo), NULL);

    if (strlen(to7_GetMemo7Label()))
        file_entry_set_entry( FILE_ENTRY(filent_memo), to7_GetMemo7Label());
    else
    {
	file_entry_set_entry( FILE_ENTRY(filent_memo), "aucune cartouche");

	if (!access("./memo7/", F_OK))
	    file_entry_set_filename( FILE_ENTRY(filent_memo), "./memo7/");
    }

    gtk_box_pack_start( GTK_BOX(vbox_memo), filent_memo, FALSE, FALSE, 0);
    gtk_widget_show(filent_memo);

    /* frame de la cassette */
    frame_cass=gtk_frame_new("prot.");
    gtk_frame_set_shadow_type( GTK_FRAME(frame_cass), GTK_SHADOW_NONE);
    gtk_frame_set_label_align( GTK_FRAME(frame_cass), 0.985, 0.0);
    label1_cass=gtk_label_new("Cassette");
    gtk_notebook_append_page( GTK_NOTEBOOK(notebook), frame_cass, label1_cass);
    gtk_widget_show(frame_cass);
    gtk_widget_show(label1_cass);

    /* boite verticale associe  la frame de la cassette */
    vbox_cass=gtk_vbox_new(FALSE, 5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox_cass), 5);
    gtk_container_add( GTK_CONTAINER(frame_cass), vbox_cass);
    gtk_widget_show(vbox_cass);

    /* premire boite horizontale de la cassette */
    hbox1_cass=gtk_hbox_new(FALSE, 10);
    gtk_box_pack_start( GTK_BOX(vbox_cass), hbox1_cass, FALSE, FALSE, 0);
    gtk_widget_show(hbox1_cass);

    /* molette du compteur de cassette */
    adj_cass = gtk_adjustment_new (0, 0, COUNTER_MAX, 1, 10, 0);
    spinner_cass = gtk_spin_button_new ( GTK_ADJUSTMENT(adj_cass), 0.5, 0);

    /* bouton protection de la cassette */
    check_button_cass=gtk_check_button_new();
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(check_button_cass), TRUE);
    gtk_signal_connect( GTK_OBJECT(check_button_cass), "toggled", GTK_SIGNAL_FUNC(toggle_check_cass), (gpointer) spinner_cass);
    gtk_box_pack_end( GTK_BOX(hbox1_cass), check_button_cass, FALSE, TRUE, 0);
    gtk_widget_show(check_button_cass);

    /* entre fichier de la cassette */
    filent_cass=file_entry_new("k7");
    gtk_signal_connect( GTK_OBJECT(filent_cass), "file_selected", GTK_SIGNAL_FUNC(read_filent_cass), (gpointer) check_button_cass);

    file_entry_set_entry( FILE_ENTRY(filent_cass), "aucune cassette");

    if (!access("./k7/", F_OK))
	file_entry_set_filename( FILE_ENTRY(filent_cass), "./k7/");

    gtk_box_pack_start( GTK_BOX(hbox1_cass), filent_cass, TRUE, TRUE, 0);
    gtk_widget_show(filent_cass);

    /* seconde boite horizontale de la cassette */
    hbox2_cass=gtk_hbox_new(FALSE, 10);
    gtk_box_pack_start( GTK_BOX(vbox_cass), hbox2_cass, FALSE, FALSE, 0);
    gtk_widget_show(hbox2_cass);

    /* molette du compteur de cassette */
    label2_cass=gtk_label_new("Compteur:");
    gtk_box_pack_start (GTK_BOX (hbox2_cass), label2_cass, FALSE, FALSE, 0);
    gtk_widget_show(label2_cass);

    /* Connecter le signal directement au spin button ne marche pas. */
    gtk_signal_connect ( GTK_OBJECT (adj_cass), "value_changed", GTK_SIGNAL_FUNC (change_counter_cass), (gpointer) spinner_cass);
    gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner_cass), FALSE);
    gtk_box_pack_start (GTK_BOX (hbox2_cass), spinner_cass, FALSE, FALSE, 0);
    gtk_widget_show(spinner_cass);

    /* bouton rembobinage */
    button_rewind=gtk_button_new_with_label("Rembobiner la cassette");
    gtk_signal_connect( GTK_OBJECT(button_rewind), "clicked", GTK_SIGNAL_FUNC(click_rewind_cass), (gpointer) spinner_cass);
    gtk_box_pack_start( GTK_BOX(hbox2_cass), button_rewind, TRUE, FALSE, 0);
    gtk_widget_show(button_rewind);

    if (disk_support)
    {
        /* frame des disquettes */
        frame_disk=gtk_frame_new("prot.");
        gtk_frame_set_shadow_type( GTK_FRAME(frame_disk), GTK_SHADOW_NONE);
        gtk_frame_set_label_align( GTK_FRAME(frame_disk), 0.985, 0.0);
        label_disk=gtk_label_new("Disquettes");
        gtk_notebook_append_page( GTK_NOTEBOOK(notebook), frame_disk, label_disk);
        gtk_widget_show(frame_disk);
        gtk_widget_show(label_disk);

        /* boite verticale associe  la frame des disquettes */
        vbox_disk=gtk_vbox_new(FALSE, 5);
        gtk_container_set_border_width( GTK_CONTAINER(vbox_disk), 5);
        gtk_container_add( GTK_CONTAINER(frame_disk), vbox_disk);
        gtk_widget_show(vbox_disk);

        /* botes horizontales des disquettes */
        for (i=0; i<NDISKS; i++)
        {
            char num[3];

            hbox_disk[i]=gtk_hbox_new(FALSE, 10);
            gtk_box_pack_start( GTK_BOX(vbox_disk), hbox_disk[i], TRUE, TRUE, 0);
            gtk_widget_show(hbox_disk[i]);

            /* boutons protection des disquettes, horrible ... */
            check_button_disk[i]=malloc(sizeof(struct dummy_pair));
            check_button_disk[i]->num=i;
            check_button_disk[i]->b=gtk_check_button_new();
            gtk_signal_connect( GTK_OBJECT(check_button_disk[i]->b), "toggled", GTK_SIGNAL_FUNC(toggle_check_disk), (gpointer) i);
            gtk_box_pack_end( GTK_BOX(hbox_disk[i]), check_button_disk[i]->b, FALSE, TRUE,0);
            gtk_widget_show(check_button_disk[i]->b);

            /* entre fichier des disquettes */
            sprintf(num, "%d:", i);
            filent_disk[i]=file_entry_new(num);
            gtk_signal_connect( GTK_OBJECT(filent_disk[i]), "file_selected", GTK_SIGNAL_FUNC(read_filent_disk), (gpointer) check_button_disk[i]);

            file_entry_set_entry( FILE_ENTRY(filent_disk[i]), "aucune disquette");

            if (!access("./disks/", F_OK))
                file_entry_set_filename( FILE_ENTRY(filent_disk[i]), "./disks/");

            gtk_box_pack_start( GTK_BOX(hbox_disk[i]), filent_disk[i], TRUE, TRUE, 0);
            gtk_widget_show(filent_disk[i]);
        }

        if (direct_disk_support)
        {
            /* bouton lecture directe */
            filentry_octo=malloc(sizeof(struct dummy_octo));

            for (i=0; i<4; i++)
            {
                if (direct_disk_support & (1<<i))
                {
                    filentry_octo->f[i]=filent_disk[i];
                    filentry_octo->b[i]=check_button_disk[i]->b;
                }
                else
                {
                    filentry_octo->f[i]=NULL;
                    filentry_octo->b[i]=NULL;
                }
            }

            button_direct=gtk_button_new_with_label("Accs direct");
            gtk_signal_connect( GTK_OBJECT(button_direct), "clicked", GTK_SIGNAL_FUNC(set_direct_disk), (gpointer) filentry_octo);
            gtk_box_pack_start( GTK_BOX(vbox_disk), button_direct, TRUE, TRUE, 0);
            gtk_widget_show(button_direct);
        }

        /* on affiche la page des disquettes au dmarrage */
        gtk_notebook_set_page( GTK_NOTEBOOK(notebook), 2);
    }
    else
    {
        /* on affiche la page des cassettes au dmarrage */
        gtk_notebook_set_page( GTK_NOTEBOOK(notebook), 1);
    }

    /* boite horizontale des boutons de sortie */
    hbox_end=gtk_hbutton_box_new();
    gtk_hbutton_box_set_layout_default(GTK_BUTTONBOX_SPREAD);
    gtk_box_pack_start( GTK_BOX(vbox_window), hbox_end, TRUE, FALSE, 0);
    gtk_widget_show(hbox_end);

    /* bouton retour */
    button_ok=gtk_button_new_with_label("OK");
    gtk_signal_connect( GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(do_exit), (gpointer) NONE);
    gtk_container_add( GTK_CONTAINER(hbox_end), button_ok);
    gtk_widget_show(button_ok);

    /* bouton quitter */
    button_quit=gtk_button_new_with_label("Quitter");
    gtk_signal_connect( GTK_OBJECT(button_quit), "clicked", GTK_SIGNAL_FUNC(question_exit), NULL);
    gtk_container_add( GTK_CONTAINER(hbox_end), button_quit);
    gtk_widget_show(button_quit);

    /* mise en place d'un hook pour assurer le retraage de la fentre
       principale de l'mulateur */
    gdk_window_add_filter(gdk_window_foreign_new(screen_win), retrace_callback, NULL);

    printf("ok\n");
}



/* ControlPanel:
 *  Affiche le panneau de contrle.
 */
void ControlPanel(void)
{
    /* Mise  jour du compteur de cassettes */
    gtk_spin_button_set_value( GTK_SPIN_BUTTON(spinner_cass), to7_GetK7Counter());

    /* affichage de la fentre principale et de ses lments */
    gtk_widget_show(window);

    /* boucle de gestion des vnements */
    gtk_main();
}
