/*
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MM          MM
 *    TTTTTTTTTTTTTT  HH          HH  OOOOOOOOOOOOOO  MMM        MMM
 *          TT        HH          HH  OO          OO  MMMM      MMMM 
 *          TT        HH          HH  OO          OO  MM MM    MM MM
 *          TT        HH          HH  OO          OO  MM  MM  MM  MM 
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM   MMMM   MM
 *          TT        HHHHHHHHHHHHHH  OO          OO  MM    MM    MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OO          OO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM
 *          TT        HH          HH  OOOOOOOOOOOOOO  MM          MM 
 *
 *                      l'mulateur Thomson TO7-70
 *
 *  Copyright (C) 1996 Sylvain Huet, 1999-2003 Eric Botcazou.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 *  Module     : linux/question.c
 *  Version    : 1.5.5
 *  Cr par   : Eric Botcazou juillet 1999
 *  Modifi par: Eric Botcazou 03/11/2003
 *
 *  Classe Question d'extension du toolkit GTK+ 1.2.x .
 */


#ifndef SCAN_DEPEND
   #include <gtk/gtkbutton.h>
   #include <gtk/gtkhbbox.h>
   #include <gtk/gtklabel.h>
   #include <gtk/gtksignal.h>
   #include <gtk/gtkvbox.h>
#endif

#include "linux/question.h"



/* partie prive de la classe Question:
 */
enum {
    CLICKED_YES_SIGNAL,
    CLICKED_NO_SIGNAL,
    LAST_SIGNAL
};

static gint question_signals[LAST_SIGNAL]={0};



static void question_class_init(QuestionClass *class)
{
    GtkObjectClass *object_class;

    object_class=(GtkObjectClass *) class;

    question_signals[CLICKED_YES_SIGNAL]=gtk_signal_new("clicked_yes", GTK_RUN_FIRST, object_class->type, GTK_SIGNAL_OFFSET(QuestionClass, clicked_yes), gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

    question_signals[CLICKED_NO_SIGNAL]=gtk_signal_new("clicked_no", GTK_RUN_FIRST, object_class->type, GTK_SIGNAL_OFFSET(QuestionClass, clicked_no), gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);

    gtk_object_class_add_signals(object_class, question_signals, LAST_SIGNAL);

    class->clicked_yes=NULL;
    class->clicked_no=NULL;
}



static void question_click_yes(GtkWidget *button, GtkWidget *quest)
{
     gtk_signal_emit(GTK_OBJECT(quest), question_signals[CLICKED_YES_SIGNAL]);

     gtk_widget_destroy(quest);

    (void) button;
}



static void question_click_no(GtkWidget *button, GtkWidget *quest)
{
     gtk_signal_emit(GTK_OBJECT(quest), question_signals[CLICKED_NO_SIGNAL]);

     gtk_widget_destroy(quest);

     (void) button;
}



static void question_init(Question *quest)
{
    GtkWidget *vbox, *button_box, *button_yes, *button_no;

    quest->window.type=GTK_WINDOW_DIALOG;
    gtk_window_set_title(&quest->window, "Thom - confirmation");
    gtk_window_set_position(&quest->window, GTK_WIN_POS_CENTER);
    gtk_signal_connect_object( GTK_OBJECT(&quest->window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(quest) );

    vbox=gtk_vbox_new(FALSE,5);
    gtk_container_set_border_width( GTK_CONTAINER(vbox), 10);
    gtk_container_add( GTK_CONTAINER(&quest->window), vbox);
    gtk_widget_show(vbox);

    quest->label=gtk_label_new("\0");
    gtk_box_pack_start( GTK_BOX(vbox), quest->label, TRUE, FALSE, 0);
    gtk_widget_show(quest->label);

    button_box=gtk_hbutton_box_new();
    gtk_box_pack_start( GTK_BOX(vbox), button_box, TRUE, FALSE, 0);
    gtk_widget_show(button_box);

    button_yes=gtk_button_new_with_label("Oui");
    gtk_container_add( GTK_CONTAINER(button_box), button_yes);
    gtk_signal_connect( GTK_OBJECT(button_yes), "clicked", GTK_SIGNAL_FUNC(question_click_yes), quest);
    gtk_widget_show(button_yes);

    button_no=gtk_button_new_with_label("Non");
    gtk_container_add( GTK_CONTAINER(button_box), button_no);
    gtk_signal_connect( GTK_OBJECT(button_no), "clicked", GTK_SIGNAL_FUNC(question_click_no), quest);
    gtk_widget_show(button_no);
}



/* partie publique de la classe Question:
 */
guint question_get_type(void)
{
    static guint quest_type = 0;

    if (!quest_type)
    {
        GtkTypeInfo quest_info={ "Question",
				sizeof(Question),
				sizeof(QuestionClass),
				(GtkClassInitFunc) question_class_init,
				(GtkObjectInitFunc) question_init,
				NULL,
				NULL,
				(GtkClassInitFunc) NULL };

        quest_type=gtk_type_unique(gtk_window_get_type(),&quest_info);
    }

    return quest_type;
}



GtkWidget *question_new(const gchar *label)
{
    Question *quest=QUESTION( gtk_type_new (question_get_type() ));

    gtk_label_set_text( GTK_LABEL(quest->label), label);

    return GTK_WIDGET(quest);
}
